// CHydro.cpp: implementation of the Hydro class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CHydro.h"
#include "constants.h"

//#include <CTaciteHydro.h>
//#include <CSyntheticHydro.h>
//#include <COlgasHydro2P.h>
//#include <COlgasHydro3P.h>
//#include <CSimpleHydro.h>
//#include <CHysiHydro.h>

/***************************************************************************************************************
*
*
***************************************************************************************************************/
//CHydro* CHydro::createHydro(enum hydroCst::modelHydro whichHydro){
//	switch(whichHydro) {
//		case hydroCst::COMP:
//			return new CTaciteHydro;
//		case hydroCst::SYNT:
//			return new CSyntheticHydro;
//		case hydroCst::OLGA2P:
//			return new COlgasHydro2P;
//		case hydroCst::OLGA3P:
//			return new COlgasHydro3P;
//		case hydroCst::SIMP:
//			return new CSimpleHydro;
//		case hydroCst::HYSI_IFP:
//		case hydroCst::HYSI_OLGA:
//			return new CHysiHydro();
//		default:
//			return NULL;
//	}
//}
/***************************************************************************************************************
*
*
***************************************************************************************************************/
//CHydro* CHydro::loadModel(CHydroData* phydroData){
//	CHydro* pHydro = NULL;
//
//	enum hydroCst::modelHydro whichModHydro = phydroData->get_hydroModel();
//	
//	pHydro = createHydro(whichModHydro);
//
//	if(pHydro)
//		pHydro->initialize(phydroData);
//	return pHydro;
//}
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** Constructor with sizes. 
	@param dim maximum number of phase
*/
CHydro::CHydro() :
	m_phaseQualifiers(NBPHASES, UNINITIALIZE),
	m_localVolumetricFractions(NBPHASES, DOUBLE_INF),
	m_flowingVolumetricFractions(NBPHASES, DOUBLE_INF),
	m_densities(NBPHASES, DOUBLE_INF),
	m_viscosities(NBPHASES, DOUBLE_INF),
	m_surfaceTensions(NBPHASES, NBPHASES, DOUBLE_INF),
	m_heatCapacities(NBPHASES, DOUBLE_INF),
	m_conductivities(NBPHASES, DOUBLE_INF),
	m_superficialVelocities(NBPHASES, DOUBLE_INF),
	m_absoluteVelocities(NBPHASES, DOUBLE_INF),
	m_absoluteVelocitiesDLocalVolumetricFractions(NBPHASES, NBPHASES, DOUBLE_INF),
	m_absoluteVelocitiesDDensities(NBPHASES, NBPHASES, DOUBLE_INF),
	m_absoluteVelocitiesDViscosities(NBPHASES, NBPHASES, DOUBLE_INF),
	m_absoluteVelocitiesDSurfaceTensions(NBPHASES, NBPHASES, NBPHASES, DOUBLE_INF),
	m_absoluteVelocitiesDBarycentricVelocity(NBPHASES, DOUBLE_INF),
	m_absoluteVelocitiesDMixtureVelocity(NBPHASES, DOUBLE_INF),
	m_localPressureDropDLocalVolumetricFractions(NBPHASES, DOUBLE_INF),
	m_localPressureDropDDensities(NBPHASES, DOUBLE_INF),
	m_localPressureDropDViscosities(NBPHASES, DOUBLE_INF),
	m_localPressureDropDSurfaceTensions(NBPHASES, NBPHASES, DOUBLE_INF),
	m_localPressureDropDBarycentricVelocity(DOUBLE_INF),
	m_localPressureDropDMixtureVelocity(DOUBLE_INF),
	m_internalFluidResistivityDLocalVolumetricFractions(NBPHASES, DOUBLE_INF),
	m_internalFluidResistivityDDensities(NBPHASES, DOUBLE_INF),
	m_internalFluidResistivityDViscosities(NBPHASES, DOUBLE_INF),
	m_internalFluidResistivityDSurfaceTensions(NBPHASES, NBPHASES, DOUBLE_INF),
	m_internalFluidResistivityDHeatCapacities(NBPHASES, DOUBLE_INF),
	m_internalFluidResistivityDConductivities(NBPHASES, DOUBLE_INF),
	m_internalFluidResistivityDBarycentricVelocity(DOUBLE_INF),
	m_effectiveLiquidViscosity(NBPHASES, DOUBLE_INF)
{
	m_nbrOfPresentPhases = INTEGER_INF;
	m_diameter = m_slopeSinus = m_roughness = m_barycentricVelocity = m_mixtureVelocity = m_localPressureDrop = m_frictionPressureDrop =
		m_gravityPressureDrop = m_internalFluidResistivity = m_slugFraction = DOUBLE_INF;
	m_flowPattern = UNDEFINED;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
CHydro::~CHydro(){
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
void CHydro::initialiseHydroOutputs(){
	m_internalFluidResistivity = m_barycentricVelocity = m_mixtureVelocity = m_localPressureDrop = m_frictionPressureDrop = m_gravityPressureDrop =
		m_slugFraction = m_localPressureDropDBarycentricVelocity = m_internalFluidResistivityDBarycentricVelocity = DOUBLE_INF;
	m_flowPattern = UNDEFINED;
	m_phaseQualifiers = TNT::Array1D<EPhases>(NBPHASES, UNINITIALIZE);
	
	for(int i = 0; i<NBPHASES; ++i){
		m_absoluteVelocities[i] = m_superficialVelocities[i] = m_localVolumetricFractions[i] = m_flowingVolumetricFractions[i] =
			m_absoluteVelocitiesDBarycentricVelocity[i] = m_localPressureDropDLocalVolumetricFractions[i] =
			m_localPressureDropDDensities[i] = m_localPressureDropDViscosities[i] = m_internalFluidResistivityDLocalVolumetricFractions[i] =
			m_internalFluidResistivityDDensities[i] = m_internalFluidResistivityDViscosities[i] = m_internalFluidResistivityDHeatCapacities[i] =
			m_internalFluidResistivityDConductivities[i] = DOUBLE_INF;
			m_effectiveLiquidViscosity[i] = DOUBLE_INF;

		for(int j = 0; j < NBPHASES; ++j){
			m_absoluteVelocitiesDLocalVolumetricFractions[i][j] = m_absoluteVelocitiesDDensities[i][j] =
				m_absoluteVelocitiesDViscosities[i][j] = m_internalFluidResistivityDSurfaceTensions[i][j] =
				m_localPressureDropDSurfaceTensions[i][j] = DOUBLE_INF;

			for(int k = 0; k < NBPHASES; ++k)
				m_absoluteVelocitiesDSurfaceTensions[i][j][k] = DOUBLE_INF;
		}
	}
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getAbsoluteVelocities() const
{
	return &m_absoluteVelocities;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getSuperficialVelocities() const
{
	return &m_superficialVelocities;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getLocalVolumetricFractions() const
{
	return &m_localVolumetricFractions;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CHydro::getBarycentricVelocity() const
{
	return m_barycentricVelocity;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CHydro::getLocalPressureDrop() const
{
	return m_localPressureDrop;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CHydro::getFrictionPressureDrop() const
{
	return m_frictionPressureDrop;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CHydro::getGravityPressureDrop() const
{
	return m_gravityPressureDrop;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CHydro::getInternalFluidResistivity() const
{
	return m_internalFluidResistivity;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
EHydroFlowPatterns CHydro::getFlowPattern() const
{
	return m_flowPattern;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CHydro::getSlugFraction() const
{
	return m_slugFraction;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getEffectiveLiquidViscosity() const
{
	return &m_effectiveLiquidViscosity;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array2D<double> const* CHydro::getAbsoluteVelocitiesDLocalVolumetricFractions() const
{
	return &m_absoluteVelocitiesDLocalVolumetricFractions;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array2D<double> const* CHydro::getAbsoluteVelocitiesDDensities() const
{
	return &m_absoluteVelocitiesDDensities;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array2D<double> const* CHydro::getAbsoluteVelocitiesDViscosities() const
{
	return &m_absoluteVelocitiesDViscosities;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array3D<double> const* CHydro::getAbsoluteVelocitiesDSurfaceTensions() const
{
	return &m_absoluteVelocitiesDSurfaceTensions;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getAbsoluteVelocitiesDBarycentricVelocity() const
{
	return &m_absoluteVelocitiesDBarycentricVelocity;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** Get derivatives of absolute velocities versus the mixture velocity */
TNT::Array1D<double> const* CHydro::getAbsoluteVelocitiesDMixtureVelocity() const
{
	return &m_absoluteVelocitiesDMixtureVelocity;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getLocalPressureDropDLocalVolumetricFractions() const
{
	return &m_localPressureDropDLocalVolumetricFractions;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getLocalPressureDropDDensities() const
{
	return &m_localPressureDropDDensities;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getLocalPressureDropDViscosities() const
{
	return &m_localPressureDropDViscosities;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array2D<double> const* CHydro::getLocalPressureDropDSurfaceTensions() const
{
	return &m_localPressureDropDSurfaceTensions;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CHydro::getLocalPressureDropDBarycentricVelocity() const
{
	return m_localPressureDropDBarycentricVelocity;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** Get derivative of the local pressure drop versus the mixture velocity */
double CHydro::getLocalPressureDropDMixtureVelocity() const
{
	return m_localPressureDropDMixtureVelocity;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getInternalFluidResistivityDLocalVolumetricFractions() const
{
	return &m_internalFluidResistivityDLocalVolumetricFractions;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getInternalFluidResistivityDDensities() const
{
	return &m_internalFluidResistivityDDensities;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getInternalFluidResistivityDViscosities() const
{
	return &m_internalFluidResistivityDViscosities;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array2D<double> const* CHydro::getInternalFluidResistivityDSurfaceTensions() const
{
	return &m_internalFluidResistivityDSurfaceTensions;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getInternalFluidResistivityDHeatCapacities() const
{
	return &m_internalFluidResistivityDHeatCapacities;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
TNT::Array1D<double> const* CHydro::getInternalFluidResistivityDConductivities() const
{
	return &m_internalFluidResistivityDConductivities;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CHydro::getInternalFluidResistivityDBarycentricVelocity() const
{
	return m_internalFluidResistivityDBarycentricVelocity;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
