// *****************************************************************************
//  
//        
// *****************************************************************************
// *****************************************************************************
#include "stdafx.h"
#include "CSafeArrayUtils.h"
#include <atlconv.h>

/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saGetDim(SAFEARRAY *rarray, TNT::Array1D<long>& nbelemPerDim){
	long dim = ::SafeArrayGetDim(rarray);
	R2PException::Utils::ThrowException(dim>0, R2PException::R2P_EBADARG, L"", L"");
	long ubound, lbound;
	HRESULT hr;

	nbelemPerDim = TNT::Array1D<long>(dim);
	for(int i=1; i<=dim; ++i){
		hr = ::SafeArrayGetLBound(rarray, i, &lbound);
		R2PException::Utils::ThrowException(SUCCEEDED(hr), hr, L"", L"");
		hr = ::SafeArrayGetUBound(rarray, i, &ubound);
		R2PException::Utils::ThrowException(SUCCEEDED(hr), hr, L"", L"");
		nbelemPerDim[i-1] = (ubound-lbound+1);
	}
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saCreate(VARTYPE vt, long* count, int dim, VARIANT& varOut){
	SAFEARRAYBOUND* rgsabound = (SAFEARRAYBOUND*)alloca(sizeof(SAFEARRAYBOUND)*dim);
	
	unsigned long nbelem = 0;
	for(int i=0; i<dim; ++i){
		rgsabound[i].lLbound = 0;
		rgsabound[i].cElements = count[i];
		nbelem += count[i];
	}
	varOut.parray = SafeArrayCreate(vt, dim, rgsabound);
	R2PException::Utils::ThrowException((varOut.parray != NULL), E_OUTOFMEMORY, L"Array Error", L"Error while allocating a %d-array of %uld elements\n", dim, nbelem);
	varOut.vt = VT_ARRAY|vt;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
VARTYPE CSAFEARRAYUTILS::get_type(SAFEARRAY* rarray){
	VARTYPE arrayType;
	AtlSafeArrayGetActualVartype(rarray, &arrayType);
	return arrayType;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saPut(SAFEARRAY *rarray, long* index, const char* data){
	VARTYPE type = get_type(rarray);
	R2PException::Utils::ThrowException(type == VT_VARIANT || type==VT_BSTR, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while inserting element. Please, contact support\n");
	BSTR tmp = NULL;
	try{
		if(data == 0)
			tmp = ::SysAllocString(L"");
		else
			tmp = A2BSTR(data); //A2BSTR does a SysAllocString
		if(type == VT_VARIANT){
			VARIANT var;
			VariantInit(&var);
			V_VT(&var) = VT_BSTR;
			//no need to sysalloc: tmp is already allocated and will be freed after
			var.bstrVal = tmp;
			saPut(rarray, index, var);
		}
		else
			saPutData(rarray, index, tmp);
	}
	catch(R2PException::CMyException& herr){
		::SysFreeString(tmp);
		throw herr;
	}
	catch(...){
		::SysFreeString(tmp);
		throw R2PException::CMyException(E_UNEXPECTED,  L"Array Error", L"Error while inserting element into array.\n");
	}
	::SysFreeString(tmp);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saPut(SAFEARRAY *rarray, long* index, const wchar_t* data){
	VARTYPE type = get_type(rarray);
	R2PException::Utils::ThrowException(type == VT_VARIANT || type==VT_BSTR, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while inserting element. Please, contact support\n");

	BSTR tmp = NULL;
	try{
		if(data == 0)
			tmp = ::SysAllocString(L"");
		else
			tmp = ::SysAllocString(data);
		if(type == VT_VARIANT){
			VARIANT var;
			VariantInit(&var);
			V_VT(&var) = VT_BSTR;
			//no need to sysalloc: tmp is already allocated and will be freed after
			var.bstrVal = tmp;
			saPut(rarray, index, var);
		}
		else
			saPutData(rarray, index, tmp);
	}
	catch(R2PException::CMyException& herr){
		::SysFreeString(tmp);
		throw herr;
	}
	catch(...){
		::SysFreeString(tmp);
		throw R2PException::CMyException(E_UNEXPECTED,  L"Array Error", L"Error while inserting element into array.\n");
	}
	::SysFreeString(tmp);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saPut(SAFEARRAY* rarray, long* index, IDispatch* data){
	VARTYPE type = get_type(rarray);
	R2PException::Utils::ThrowException(type == VT_VARIANT || type==VT_DISPATCH, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while inserting element. Please, contact support\n");

	if(type == VT_VARIANT){
		VARIANT var;
		VariantInit(&var);
		V_VT(&var) = VT_DISPATCH;
		var.pdispVal = data;
		saPut(rarray, index, var);
		//no need to addref: the safe array put element do it for us
		//Do not clean Variant. If so, we've had to addref.
	}
	else
		saPutData(rarray, index, data);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saPut(SAFEARRAY* rarray, long* index, const std::wstring& data){
	saPut(rarray, index, data.c_str());
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saPut(SAFEARRAY* rarray, long* index, const double& data){
	VARTYPE type = get_type(rarray);
	R2PException::Utils::ThrowException(type == VT_VARIANT || type==VT_R8, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while inserting element. Please, contact support\n");

	if(type == VT_VARIANT){
		VARIANT var;
		VariantInit(&var);
		V_VT(&var) = VT_R8;
		var.dblVal = data;
		saPut(rarray, index, var);
		VariantClear(&var);
	}
	else
		saPutData(rarray, index, (void*)&data);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saPut(SAFEARRAY* rarray, long* index, const long& data){
	VARTYPE type = get_type(rarray);
	R2PException::Utils::ThrowException(type == VT_VARIANT || type==VT_I4, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while inserting element. Please, contact support\n");

	if(type == VT_VARIANT){
		VARIANT var;
		VariantInit(&var);
		V_VT(&var) = VT_I4;
		var.lVal = data;
		saPut(rarray, index, var);
		VariantClear(&var);
	}
	else
		saPutData(rarray, index, (void*)&data);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saPut(SAFEARRAY* rarray, long* index, VARIANT& data){
	saPutData(rarray, index, &data);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saGet(SAFEARRAY *rarray, long* index, char** data){
	/*
	No Need try/catch block
	When saGet or saGetData methods fails: the out parameter is cleaned
	*/
	
	USES_CONVERSION;
	BSTR bstrRes = NULL;
	char* tmp;
	VARTYPE type = get_type(rarray);

	R2PException::Utils::ThrowException(type == VT_VARIANT || type==VT_BSTR, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while getting element. Please, contact support\n");

	if(type == VT_VARIANT){
		VARIANT var;
		VariantInit(&var);
		saGet(rarray, index, &var);
		R2PException::Utils::ThrowException(var.vt == VT_BSTR, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while getting element. Please, contact support\n");
		bstrRes = var.bstrVal;
		// no need to clear variant: there is a sysfreestring after
	}
	else
		saGetData(rarray, index, &bstrRes);
	
	//OLE2A method uses alloca method: value is valid within the scope
	tmp = OLE2A(bstrRes);
	size_t size = strlen(tmp)+1;
	*data = new char[size];
	strcpy_s(*data, size, tmp);
	::SysFreeString(bstrRes);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saGet(SAFEARRAY *rarray, long* index, wchar_t** data){
	VARTYPE type = get_type(rarray);
	R2PException::Utils::ThrowException(type == VT_VARIANT || type==VT_BSTR, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while getting element. Please, contact support\n");
	BSTR bstrRes = NULL;
	
	if(type == VT_VARIANT){
		VARIANT var;
		VariantInit(&var);
		saGet(rarray, index, &var);
		R2PException::Utils::ThrowException(var.vt == VT_BSTR, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while getting element. Please, contact support\n");
		bstrRes = var.bstrVal;
		// no need to clear variant: there is a sysfreestring after
	}
	else
		saGetData(rarray, index, &bstrRes);

	size_t len = ::SysStringLen(bstrRes)+1;
	*data = new wchar_t[len];
	wcscpy_s(*data, len, bstrRes);
	::SysFreeString(bstrRes);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saGet(SAFEARRAY* rarray, long* index, std::wstring* data){
	wchar_t* Res;
	saGet(rarray, index, &Res);
	*data = Res;
	delete [] Res;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saGet(SAFEARRAY* rarray, long* index, IDispatch** data){
	VARTYPE type = get_type(rarray);
	R2PException::Utils::ThrowException(type == VT_VARIANT || type==VT_DISPATCH, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while getting element. Please, contact support\n");
	if(type == VT_VARIANT){
		VARIANT var;
		VariantInit(&var);
		saGet(rarray, index, &var);
		R2PException::Utils::ThrowException(var.vt == VT_DISPATCH, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while getting element. Please, contact support\n");
		*data = var.pdispVal;
		// no need to clear variant: if we do, we need to call addref
	}
	else
		saGetData(rarray, index, data);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saGet(SAFEARRAY* rarray, long* index, double* data){
	VARTYPE type = get_type(rarray);
	R2PException::Utils::ThrowException(type == VT_VARIANT || type==VT_R8, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while getting element. Please, contact support\n");
	if(type == VT_VARIANT){
		VARIANT var;
		VariantInit(&var);
		saGet(rarray, index, &var);
		R2PException::Utils::ThrowException(var.vt == VT_R8, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while getting element. Please, contact support\n");
		*data = var.dblVal;
		VariantClear(&var);
	}
	else
		saGetData(rarray, index, data);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saGet(SAFEARRAY* rarray, long* index, long* data){
	VARTYPE type = get_type(rarray);
	R2PException::Utils::ThrowException(type == VT_VARIANT || type==VT_I4, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while getting element. Please, contact support\n");
	if(type == VT_VARIANT){
		VARIANT var;
		VariantInit(&var);
		saGet(rarray, index, &var);
		R2PException::Utils::ThrowException(var.vt == VT_I4, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while getting element. Please, contact support\n");		
		*data = var.lVal;
		VariantClear(&var);
	}
	else
		saGetData(rarray, index, data);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saGet(SAFEARRAY* rarray, long* index, VARIANT* data){
	saGetData(rarray, index, data);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saGet(SAFEARRAY* rarray, long* index, bool* data){
	VARIANT_BOOL val;
	saGetData(rarray, index, &val);
	*data = (val == VARIANT_TRUE);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saPut(SAFEARRAY* rarray, long* index, const bool& data){
	VARTYPE type = get_type(rarray);
	R2PException::Utils::ThrowException(type==VT_BOOL, E_INVALIDARG, L"Array Error", L"Invalid type between Array Type and element type while inserting element. Please, contact support\n");

	VARIANT_BOOL val = (data == true)? VARIANT_TRUE: VARIANT_FALSE;
	saPutData(rarray, index, &val);
}


