// CCapeBadCOParameter.h: interface for the ECapeBadCOParameter class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(__CCAPEBADCOPARAMETER_H__)
#define __CCAPEBADCOPARAMETER_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include "CCapeData.h"
#include "stdafx.h"

namespace R2PException{
	/// <summary>Exception handling class: A parameter, which is an object from the Parameter Common Interface, has an invalid status</summary> 
	class CCapeBadCOParameter: public CCapeData
	{
	public:
		/// <summary>Constructor</summary>
		/// <param name="hr">legacy error code</param>
		/// <param name="desc">default error description</param>
		CCapeBadCOParameter(HRESULT hr, const std::wstring& desc = L"");

		/// <summary>Constructor: create a CCapeBadCOParameter object from a CAPE-OPEN component</summary>
		/// <param name="pDisp">pointer to a CAPE-OPEN component which have to expose ECapeBadCOParameter interface</param>
		CCapeBadCOParameter(IDispatch* pDisp);

		/// <summary>Destructor</summary>
		~CCapeBadCOParameter();

		/// <summary>set error specific class attribute</summary>
		/// <param name="ParamName">Name of parameter that throw error</param>
		/// <returns>void</returns>
		void set_parameterName(const std::wstring& ParamName);

		/// <summary>set error specific class attribute</summary>
		/// <param name="theParam">pointer to Parameter interface</param>
		/// <returns>void</returns>
		void set_parameter(const IDispatch* theParam);

		/// <summary>get error specific class attribute</summary>
		/// <param name="ParamName">Name of parameter that throw error</param>
		/// <returns>void</returns>
		void get_parameterName(std::wstring& ParamName);

		/// <summary>get error specific class attribute</summary>
		/// <param name="theParam">pointer to Parameter interface</param>
		/// <returns>void</returns>
		/// <remarks>This method calls AddRef() on the ICapeParameter interface. Be sure that client will release it.</remarks>
		void get_parameter(IDispatch** theParam);

	protected:
		/// <summary>The name of the CO parameter</summary>
		std::wstring _parameterName;
		/// <summary>The parameter</summary>
		IDispatch* _parameter;
	};
}
#endif // __CCAPEBADCOPARAMETER_H__
