// CCapeException.h: interface for the ECapeUser class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(__CCAPEUSER_H__)
#define __CDESCERROR_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include "stdafx.h"

namespace R2PException{
	// ======================================================================================
	/// <summary>CapeOpen Exception handling class: The base class of the CO errors hierarchy. It defines the minimum state of a CO error</summary>
	/// <remarks>This is an abstract class. No real error can be raised from this class</remarks>
	class CCapeUser
	{
	public:

		/// <summary>Constructor</summary>
		/// <param name="hr">legacy error code</param>
		/// <param name="desc">default error description</param>
		CCapeUser(HRESULT hr, const std::wstring& desc = L"");

		/// <summary>Constructor: create a CCapeUser object from a CAPE-OPEN component</summary>
		/// <param name="pDisp">pointer to a CAPE-OPEN component which have to expose ECapeUser interface</param>
		CCapeUser(IDispatch* pDisp);
		
		/// <summary>Destructor</summary>
		virtual ~CCapeUser() = 0;

		/// <summary>Get legacy error code</summary>
		/// <param name="code">error code</param>
		/// <returns>void</returns>
		void get_code(long& code);

		/// <summary>Get description</summary>
		/// <param name="description">description</param>
		/// <returns>void</returns>
		void get_description(std::wstring& description);

		/// <summary>add description</summary>
		/// <param name="description">description</param>
		/// <returns>void</returns>
		void add_description(const wchar_t* description);

		/// <summary>Get scope</summary>
		/// <param name="scope">scope</param>
		/// <returns>void</returns>
		void get_scope(std::wstring& scope);

		/// <summary>Get the name of the interface where the error is thrown</summary>
		/// <param name="interfaceName">name of the interface</param>
		/// <returns>void</returns>
		void get_interfaceName(std::wstring& interfaceName);

		/// <summary>Get the name of the operation where the error is thrown</summary>
		/// <param name="operation">name of the operation</param>
		/// <returns>void</returns>
		void get_operation(std::wstring& operation);

		/// <summary>Get a string where more information on the error can be found</summary>
		/// <param name="moreInfo">string of the information</param>
		/// <returns>void</returns>
		void get_moreInfo(std::wstring& moreInfo);

		/// <summary>Set description</summary>
		/// <param name="description">description</param>
		/// <returns>void</returns>
		void set_description(const std::wstring& description);

		/// <summary>Set scope</summary>
		/// <param name="scope">scope</param>
		/// <returns>void</returns>
		void set_scope(const std::wstring& scope);

		/// <summary>Set the name of the interface where the error is thrown</summary>
		/// <param name="interfaceName">name of the interface</param>
		/// <returns>void</returns>
		void set_interfaceName(const std::wstring& interfaceName);

		/// <summary>Set the name of the operation where the error is thrown</summary>
		/// <param name="operation">name of the operation</param>
		/// <returns>void</returns>
		void set_operation(const std::wstring& operation);

		/// <summary>Set a string where more information on the error can be found</summary>
		/// <param name="moreInfo">string of the information</param>
		/// <returns>void</returns>
		void set_moreInfo(const std::wstring& moreInfo);

		/// <summary>Get description</summary>
		/// <param name="desc">brief description</param>
		/// <returns>void</returns>
		void get_name(std::wstring& name);

		/// <summary>Set description</summary>
		/// <param name="desc">brief description</param>
		/// <returns>void</returns>
		void set_name(const std::wstring& name);

	protected:
		/// <summary>A short description of the error</summary>
		std::wstring _name;

		/// <summary>Legacy error code</summary>
		long _code;

		/// <summary>The description of the error</summary>
		std::wstring _description;

		/// <summary>The scope of the error</summary>
		std::wstring _scope;

		/// <summary>The name of the interface where the error is thrown</summary>
		std::wstring _interfaceName;

		/// <summary>The name of the operation where the error is thrown</summary>
		std::wstring _operation;

		/// <summary>An URL to a page, document, web site, ... where more information on the error can be found</summary>
		std::wstring _moreInfo;
	};
}
#endif // __CDESCERROR_H__
