// CDataPack.h: interface for the CDataPack class.
//
/// /// /// /// /// /// /// /// /// /// /// /// /// /// /// /// /// /// /// /// /// /// /// /

#if !defined(AFX_CDATAPACK_H__57E519E7_D26E_4EAB_AE8C_9F8CCDEC010D__INCLUDED_)
#define AFX_CDATAPACK_H__57E519E7_D26E_4EAB_AE8C_9F8CCDEC010D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include <tnt.h>
#include <comdef.h>
#include "CSafeArrayUtils.h"
#include <algorithm>
#include "CCOMUtils.h"

/** CDATAPACK class: class for type conversion

@author P. ROUX- TINA Project - Institut Franais du Ptrole - 2005 */
// ======================================================================================

/// <summary>useful class to convert various type into its counterpart such as BSTR to std::wstring or array of "any type" to tnt::array of "any type"</summary>
class CDATAPACK{
private:
	//typedef double (*fct)(double);

	/// <summary>Copy an array 1D of T into a safearray contained in a variant</summary>
	/// <param name="array_in">array of T to convert</param>
	/// <param name="var_out">variant of T</param>
	/// <param name="vt">type of safe array element</param>
	/// <param name="len">size of array</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	template<class T> static inline void CopyArray1D2Variant(const T& array_in, VARIANT& var_out, VARTYPE vt, long len) throw(...);

	/// <summary>Convert an Array1D of T into a safearray contained in a variant</summary>
	/// <param name="array_in">array of T to convert</param>
	/// <param name="var_out">variant of T</param>
	/// <param name="vt">type of safe array element</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	template<class T> static inline void Array2Variant(const TNT::Array1D<T>& array_in, VARIANT& var_out, VARTYPE vt) throw(...);

	/// <summary>Convert an Array2D of T into a safearray contained in a variant</summary>
	/// <param name="array_in">array of T to convert</param>
	/// <param name="var_out">variant of T</param>
	/// <param name="vt">type of safe array element</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	template <class T> static inline void Array2Variant(const TNT::Array2D<T>& array_in, VARIANT& var_out, VARTYPE vt/*, void (*function)(const T*, T*&)*/) throw(...);

	/// <summary>Convert a SAFEARRAY of T containing in a VARIANT into an TNT::Array1D<T></summary>
	/// <param name="var_in">variant of T to convert</param>
	/// <param name="array_out">array of T</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	template<class T> static inline void Variant2Array(const VARIANT& var_in, TNT::Array1D<T>& array_out, VARTYPE vt) throw(...);

	/// <summary>Convert a SAFEARRAY of T containing in a VARIANT into an TNT::Array2D<T></summary>
	/// <param name="var_in">variant of T to convert</param>
	/// <param name="array_out">array of T</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	template<class T> static inline void Variant2Array(const VARIANT& var_in, TNT::Array2D<T>& array_out, VARTYPE vt) throw(...);


public:

	/// <summary>Convert an Array1D of long into a safearray of long</summary>
	/// <param name="arrayLng_in">array of long to convert</param>
	/// <param name="varLng_out">variant of long</param>
	/// <param name="isVariant">define if the array is of type variant of long</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	static void Array2Variant(const TNT::Array1D<long>& arrayLng_in, VARIANT& varLng_out, bool isVariant = false) throw(...);
	
	/// <summary>Convert an Array1D of double into a safearray of double</summary>
	/// <param name="arrayDbl_in">array of double to convert</param>
	/// <param name="varDbl_out">variant of double</param>
	/// <param name="isVariant">define if the array is of type variant of double</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	static void Array2Variant(const TNT::Array1D<double>& arrayDbl_in, VARIANT& varDbl_out, bool isVariant = false) throw(...);
	
	/// <summary>Convert an Array1D of char* into a safearray of BSTR</summary>
	/// <param name="arrayChar_in">array of char* to convert</param>
	/// <param name="varChar_out">variant of char*</param>
	/// <param name="isVariant">define if the array is of type variant of BSTR</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	static void Array2Variant(const TNT::Array1D<char*>& arrayChar_in, VARIANT& varChar_out, bool isVariant = false) throw(...);

	/// <summary>Convert an Array1D of std::wstring into a safearray of BSTR</summary>
	/// <param name="arrayChar_in">array of std::wstring to convert</param>
	/// <param name="varChar_out">variant of std::wstring</param>
	/// <param name="isVariant">define if the array is of type variant of BSTR</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	static void Array2Variant(const TNT::Array1D<std::wstring>& arrayChar_in, VARIANT& varChar_out, bool isVariant = false) throw(...);

	/// <summary>Convert an Array2D of long into a safearray of long</summary>
	/// <param name="arrayLng_in">array of long to convert</param>
	/// <param name="varLng_out">variant of long</param>
	/// <param name="isVariant">define if the array is of type variant of long</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	static void Array2Variant(const TNT::Array2D<long>& arrayLng_in, VARIANT& varLng_out, bool isVariant = false) throw(...);
	
	/// <summary>Convert an Array2D of double into a safearray of double</summary>
	/// <param name="arrayDbl_in">array of double to convert</param>
	/// <param name="varDbl_out">variant of double</param>
	/// <param name="isVariant">define if the array is of type variant of double</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	static void Array2Variant(const TNT::Array2D<double>& arrayDbl_in, VARIANT& varDbl_out, bool isVariant = false/*, void (*function)(const double*, double*&) = &CCOMUtils::nop*/) throw(...);
	
	/// <summary>Convert an Array2D of char* into a safearray of BSTR</summary>
	/// <param name="arrayChar_in">array of char* to convert</param>
	/// <param name="varChar_out">variant of char*</param>
	/// <param name="isVariant">define if the array is of type variant of BSTR</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	static void Array2Variant(const TNT::Array2D<char*>& arrayChar_in, VARIANT& varChar_out, bool isVariant = false) throw(...);

	/// <summary>Convert an Array2D of std::wstring into a safearray of BSTR</summary>
	/// <param name="arrayChar_in">array of std::wstring to convert</param>
	/// <param name="varChar_out">variant of std::wstring</param>
	/// <param name="isVariant">define if the array is of type variant of BSTR</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	static void Array2Variant(const TNT::Array2D<std::wstring>& arrayChar_in, VARIANT& varChar_out, bool isVariant = false) throw(...);

	/// <summary>Convert an Array2D of IDispatch into a safearray of IDispatch</summary>
	/// <param name="arrayDisp_in">array of std::wstring to convert</param>
	/// <param name="varDisp_out">variant of std::wstring</param>
	/// <param name="isVariant">define if the array is of type variant of IDispatch</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	static void Array2Variant(const TNT::Array2D<IDispatch*>& arrayDisp_in, VARIANT& varDisp_out, bool isVariant = false) throw(...);

	/// <summary>Convert an Array1D of IDispatch into a safearray of IDispatch</summary>
	/// <param name="arrayDisp_in">array of std::wstring to convert</param>
	/// <param name="varDisp_out">variant of std::wstring</param>
	/// <param name="isVariant">define if the array is of type variant of IDispatch</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	static void Array2Variant(const TNT::Array1D<IDispatch*>& arrayDisp_in, VARIANT& varDisp_out, bool isVariant = false) throw(...);

	/// <summary>Convert an array (C syntax) into a safearray of BSTR</summary>
	/// <param name="array_in">array of wchar_t* to convert</param>
	/// <param name="var_out">variant of BSTR</param>
	/// <param name="len">length of array</param>
	/// <param name="isVariant">define if the array is of type variant of BSTR</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return VT_EMPTY variant if array is empty: you have to check it</remarks>
	static void Array2Variant(const wchar_t* array_in[], VARIANT& var_out, size_t len, bool isVariant = false) throw(...);
	
	/// <summary>Convert a one-dimensional array of BSTR into an Array1D of std::wstring</summary>
	/// <param name="varBSTR_in">variant to convert</param>
	/// <param name="arrayWStr_out">array1D</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	static void Variant2Array(const VARIANT& varBSTR_in, TNT::Array1D<std::wstring>& arrayWStr_out) throw(...);

	/// <summary>Convert a two-dimensional array of BSTR into an Array2D of std::wstring</summary>
	/// <param name="var_in">variant to convert</param>
	/// <param name="array_out">array2D</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	static void Variant2Array(const VARIANT& var_in, TNT::Array2D<std::wstring>& array_out) throw(...);

	/// <summary>Convert a one-dimensional array of IDispatch into an Array1D of IDispatch
	/// <para> WARN: this method increments reference for each element: you have to release it by yourself</para>
	/// </summary>
	/// <param name="var_in">variant to convert</param>
	/// <param name="array_out">array1D</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	static void Variant2Array(const VARIANT& var_in, TNT::Array1D<IDispatch*>& array_out) throw(...);

	/// <summary>Convert a two-dimensional array of IDispatch into an Array2D of IDispatch
	/// <para> WARN: this method increments reference for each element: you have to release it by yourself</para>
	/// </summary>
	/// <param name="var_in">variant to convert</param>
	/// <param name="array_out">array1D</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	static void Variant2Array(const VARIANT& var_in, TNT::Array2D<IDispatch*>& array_out) throw(...);

	/// <summary>Convert a one-dimensional array of double into an Array1D of double</summary>
	/// <param name="varDbl_in">variant to convert</param>
	/// <param name="arrayDbl_out">array1D</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	static void Variant2Array(const VARIANT& varDbl_in, TNT::Array1D<double>& arrayDbl_out) throw(...);

	/// <summary>Convert a double dimension array of double into an Array2D of double</summary>
	/// <param name="varDbl_in">variant to convert</param>
	/// <param name="arrayDbl_out">array2D</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	static void Variant2Array(const VARIANT& varDbl_in, TNT::Array2D<double>& arrayDbl_out) throw(...);

	/// <summary>Convert a one-dimensional array of BOOL into an Array1D of bool</summary>
	/// <param name="varBool_in">variant to convert</param>
	/// <param name="arrayBool_out">array1D</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	static void Variant2Array(const VARIANT& varBool_in, TNT::Array1D<bool>& arrayBool_out) throw(...);

	/// <summary>Convert a two-dimensional array of BOOL into an Array2D of bool</summary>
	/// <param name="varBool_in">variant to convert</param>
	/// <param name="arrayBool_out">array2D</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	static void Variant2Array(const VARIANT& varBool_in, TNT::Array2D<bool>& arrayBool_out) throw(...);

	/// <summary>Convert a one-dimensional array of long into an Array1D of long</summary>
	/// <param name="var_in">variant to convert</param>
	/// <param name="array_out">array1D</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	static void Variant2Array(const VARIANT& var_in, TNT::Array1D<long>& array_out) throw(...);

	/// <summary>Convert a two-dimensional array of long into an Array2D of long</summary>
	/// <param name="var_in">variant to convert</param>
	/// <param name="array_out">array2D</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>May return empty array if Variant is of type VT_EMPTY: you have to check it</remarks>
	static void Variant2Array(const VARIANT& var_in, TNT::Array2D<long>& array_out) throw(...);
	
	/// <summary>Convert a std::wstring into a BSTR</summary>
	/// <param name="ws_in">std::wstring to convert</param>
	/// <param name="bstr_out">BSTR</param>
	/// <returns>void</returns>
	/// <remarks>May return NULL bstr</remarks>
	static void wstring2BSTR(const std::wstring& ws_in, BSTR& bstr_out);

	/// <summary>Convert a whar_t string into a BSTR</summary>
	/// <param name="wchar_in">wchar string to convert</param>
	/// <param name="bstr_out">BSTR</param>
	/// <returns>void</returns>
	/// <remarks>May return NULL bstr</remarks>
	static void wchar2BSTR(const wchar_t* wsIn, BSTR& bstrOut);

	/// <summary>Convert a string into a wild char string</summary>
	/// <param name="a_Src">string to convert</param>
	/// <param name="a_Dest">wide char string</param>
	/// <returns>void</returns>
	/// <remarks>May return an empty string</remarks>
	static void String2WString(const std::string& a_Src, std::wstring& a_Dest);

	/// <summary>Convert a std::wstring into a  string</summary>
	/// <param name="a_Src">std::wstring to convert</param>
	/// <param name="a_Dest">string</param>
	/// <returns>void</returns>
	/// <remarks>May return an empty string</remarks>
	static void WString2String(const std::wstring& a_Src, std::string& a_Dest);

	/// <summary>Convert a string into a wild char string</summary>
	/// <param name="a_Src">string to convert</param>
	/// <param name="a_Dest">wide char string</param>
	/// <returns>void</returns>
	/// <remarks>May return an empty string</remarks>
	static void String2WString(const char* a_Src, std::wstring& a_Dest);

	/// <summary>Convert a bstr string into a wild char string</summary>
	/// <param name="a_Src">bstr string to convert</param>
	/// <param name="a_Dest">wide char string</param>
	/// <returns>void</returns>
	/// <remarks>May return an empty string</remarks>
	/// <exception>Any</exception>
	static void BSTR2WString(const BSTR a_Src, std::wstring& a_Dest) throw();

	/// <summary>Convert an array of wstring into an array of double value</summary>
	/// <param name="a_Src">array of string to convert</param>
	/// <param name="a_Dest">array ot double to get back</param>
	/// <returns>void</returns>
	/// <exception>Any</exception>
	static void wstring2double(const TNT::Array1D<std::wstring>& a_Src, TNT::Array1D<double>& a_Dest) throw();
};

/****************************************************************************************************************************
*		inline & template methods						
*
****************************************************************************************************************************/
template <class T> void CDATAPACK::CopyArray1D2Variant(const T& array_in, VARIANT& var_out, VARTYPE vt, long len){
	if(len != 0){
		try{
			V_VT(&var_out) = VT_ARRAY|vt;
			CSAFEARRAYUTILS::saCreate(vt, &len, 1, var_out);

			for(long i=0; i<len; ++i)
				CSAFEARRAYUTILS::saPut(var_out.parray, &i, array_in[(int)i]);
		}
		catch(R2PException::CMyException& herr){
			VariantClear(&var_out);
			throw herr;
		}
		catch(...){
			VariantClear(&var_out);
			throw R2PException::CMyException(E_UNEXPECTED, L"UnExpected exception", L"");
		}
	}
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
template <class T> void CDATAPACK::Array2Variant(const TNT::Array1D<T>& array_in, VARIANT& var_out, VARTYPE vt)
{
	if(var_out.vt != VT_EMPTY)
		VariantClear(&var_out);

	VariantInit(&var_out);

	CopyArray1D2Variant(array_in, var_out, vt, array_in.dim());
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
template <class T> void CDATAPACK::Array2Variant(const TNT::Array2D<T>& array_in, VARIANT& var_out, VARTYPE vt/*, void (*function)(const T*, T*&)*/)
{
	if(var_out.vt != VT_EMPTY)
		VariantClear(&var_out);

	VariantInit(&var_out);

	HRESULT hr = E_FAIL;


	int size1 = array_in.dim1();
	int size2 = array_in.dim2();
	long tabsize[2] = {size1, size2};
	if((size1 != 0) && (size2 != 0)){
		try{
			V_VT(&var_out) = VT_ARRAY|vt;

			CSAFEARRAYUTILS::saCreate(vt, tabsize, 2, var_out);
			long index[2];
			for(long i=0; i<size1; ++i){
				index[0] = i;
				for(long j=0; j<size2; ++j){
					index[1] = j;
					CSAFEARRAYUTILS::saPut(var_out.parray, index, array_in[i][j]);
				}
			}
		}
		catch(R2PException::CMyException& herr){
			VariantClear(&var_out);
			throw herr;
		}
		catch(...){
			VariantClear(&var_out);
			std::wstring errmsg = L"Creating a 2-D array of size ";
			wchar_t tmp[100];
			_ltow_s(size1*size2, tmp, 100, 10);
			errmsg +=  tmp;
			errmsg += L"\n";
			throw R2PException::CMyException(E_UNEXPECTED, L"Array error", errmsg.c_str());
		}
	}
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
template<class T> void CDATAPACK::Variant2Array(const VARIANT& var_in, TNT::Array1D<T>& array_out, VARTYPE vt){
	if (V_VT(&var_in) == (VT_ARRAY|vt) || V_VT(&var_in) == (VT_ARRAY|VT_VARIANT)){
		long ub = 0;
		long lb = 0;
		HRESULT	hr = E_FAIL;
		
		try{
			unsigned int nbdim = SafeArrayGetDim(var_in.parray);
			R2PException::Utils::ThrowException((nbdim == 1), E_INVALIDARG, L"Array error", L"Try to convert a %d-dimensional array into 1-dimensional array. Please contact support\n", nbdim);
			
			hr = SafeArrayGetLBound(var_in.parray, 1, &lb);
			R2PException::Utils::ThrowException(SUCCEEDED(hr), hr, L"Array error", L"");
			hr = SafeArrayGetUBound(var_in.parray, 1, &ub);
			R2PException::Utils::ThrowException(SUCCEEDED(hr), hr, L"Array error", L"");
			long size = ub-lb+1;
			R2PException::Utils::ThrowException(size > 0, E_INVALIDARG, L"Array Error", L"Please contact support\n");
		
			TNT::Array1D<T> tntArray(size);
			for(long i=lb; i<ub+1; ++i)
				CSAFEARRAYUTILS::saGet(var_in.parray, &i, &tntArray[int(i)]);
				
			array_out = tntArray;
		}
		catch(R2PException::CMyException& herr){
			array_out = TNT::Array1D<T>(0);
			throw herr;
		}
		catch(...){
			array_out = TNT::Array1D<T>(0);
			std::wstring errmsg = L"Creating a 1-D array of size ";
			wchar_t tmp[100];
			if (ub != 0 && lb != 0){
				wchar_t tmp[100];
				_ltow_s(ub-lb+1, tmp, 100, 10);
			}
			else
				wcscpy_s(tmp, 10, L"NaN");
			errmsg +=  tmp;
			errmsg += L"\n";
			throw R2PException::CMyException(E_UNEXPECTED, L"Array error", errmsg.c_str());
		}
	}
	else{
		if (V_VT(&var_in) == VT_EMPTY){
			TNT::Array1D<T> retVal (0);
			array_out = retVal;
		}
		else
			throw R2PException::CMyException(E_INVALIDARG, L"Array error", L"Please contact support.\n");
	}
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
template<class T> void CDATAPACK::Variant2Array(const VARIANT& var_in, TNT::Array2D<T>& array_out, VARTYPE vt){
	if (V_VT(&var_in) == (VT_ARRAY|vt) || V_VT(&var_in) == (VT_ARRAY|VT_VARIANT))
	{
		// DEFINE ARRAY DIMENSION
		int dim1 = 0, dim2 = 0;
		long ub = 0;
		HRESULT hr = E_FAIL;
		try{
			unsigned int nbdim = SafeArrayGetDim(var_in.parray);
			R2PException::Utils::ThrowException((nbdim == 2), E_INVALIDARG, L"Array error", L"Try to convert a %d-dimensional array into 2-dimensional array. Please contact support\n", nbdim);

			hr = SafeArrayGetUBound(var_in.parray, 1, &ub);
			R2PException::Utils::ThrowException(SUCCEEDED(hr), hr, L"Array error", L"");
			
			dim1 = int(ub+1);

			hr = SafeArrayGetUBound(var_in.parray, 2, &ub);
			R2PException::Utils::ThrowException(SUCCEEDED(hr), hr, L"Array error", L"");

			dim2 = int(ub+1);

			//GET ARRAY VALUES
			array_out = TNT::Array2D<T>(dim1, dim2);
			long index[2];
			for(long i=0; i<dim1; ++i){
				index[0] = i;
				for(long j=0; j<dim2; ++j){
					index[1] = j;
					T val;
					CSAFEARRAYUTILS::saGet(var_in.parray, index, &val);
					array_out[(int)i][(int)j] = val;
				}
			}
		}
		catch(R2PException::CMyException& herr){
			array_out = TNT::Array2D<T>(0,0);
			throw herr;
		}
		catch(...){
			array_out = TNT::Array2D<T>(0,0);
			wchar_t tmp[100];
			if (dim1 != 0 && dim2 != 0)
				_ltow_s(dim1*dim2, tmp, 100, 10);
			else
				wcscpy_s(tmp, 10, L"NaN");
			throw R2PException::CMyException(E_UNEXPECTED, L"Array error", L"Creating a 2-D array of size %s.\n", tmp);
		}
	}
	else{
		if (V_VT(&var_in) == VT_EMPTY){
			array_out = TNT::Array2D<T>(0,0);
		}
		else
			throw R2PException::CMyException(E_INVALIDARG, L"Array error", L"Please contact support\n");
	}
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/


#endif // !defined(AFX_CDATAPACK_H__57E519E7_D26E_4EAB_AE8C_9F8CCDEC010D__INCLUDED_)

