// CCapePersistenceNotFound.h: interface for the ECapePersistenceNotFound class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(__CCAPEPERSISTENCENOTFOUND_H__)
#define __CCAPEPERSISTENCENOTFOUND_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include "CCapePersistence.h"
#include "stdafx.h"

namespace R2PException{
	/// <summary>Exception handling class: The requested object, table, or something else within the persistence system was not found</summary> 
	class CCapePersistenceNotFound: public CCapePersistence
	{
	public:
		/// <summary>Constructor</summary>
		/// <param name="hr">legacy error code</param>
		/// <param name="desc">default error description</param>
		CCapePersistenceNotFound(HRESULT hr, const std::wstring& desc = L"");

		/// <summary>Constructor: create a CCapePersistenceNotFound object from a CAPE-OPEN component</summary>
		/// <param name="pDisp">pointer to a CAPE-OPEN component which have to expose ECapePersistenceNotFound interface</param>
		CCapePersistenceNotFound(IDispatch* pDisp);

		/// <summary>Destructor</summary>
		~CCapePersistenceNotFound();

		/// <summary>set error specific class attribute</summary>
		/// <param name="iName">name of item</param>
		/// <returns>void</returns>
		void set_itemName(const std::wstring& iName);

		/// <summary>get error specific class attribute</summary>
		/// <param name="iName">name of item</param>
		/// <returns>void</returns>
		void get_itemName(std::wstring& iName);

	protected:
		/// <summary>The name of the item which has not been found</summary>
		std::wstring _itemName;

	};
}
#endif // __CCAPEPERSISTENCENOTFOUND_H__
