#include "CCapeUser.h"
#include "CDATAPACK.h"

namespace R2PException{
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
CCapeUser::CCapeUser(HRESULT hr, const std::wstring& desc){
	_code = hr;
	_description = desc;
	_scope = L"";
	_interfaceName = L"";
	_operation = L"";
	_moreInfo = L"";
	_name = L"";
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
CCapeUser::CCapeUser(IDispatch* pDisp){
	ECapeRoot* pECRoot = NULL;
	HRESULT hres = pDisp->QueryInterface(IID_ECapeRoot,(void**)&pECRoot);
	_description = L"";
	if(FAILED(hres)){
		_name = L"CAPE-OPEN interface not implemented.";
		_description = L"WARNING: Cannot access ECapeRoot error interface from a third CAPE_OPEN component.\n";
	}
	else{
		BSTR bstrname = NULL;
		hres = pECRoot->get_name(&bstrname);
		CDATAPACK::BSTR2WString(bstrname, _name);
		::SysFreeString(bstrname);
	}
	if(pECRoot) pECRoot->Release();

	ECapeUser* pECUsr = NULL;
	hres = pDisp->QueryInterface(IID_ECapeUser,(void**)&pECUsr);
	if(FAILED(hres)){
		_description += L"WARNING: Cannot access ECapeUser error interface from a third CAPE_OPEN component.\n";
		_code = 0;
		_scope = L"";
		_interfaceName = L"";
		_operation = L"";
		_moreInfo = L"";
	}
	else{
		BSTR bstrdesc = NULL, bstrscope = NULL, bstrinterfacename = NULL, bstroperation = NULL, bstrmoreinfo = NULL;
		
		hres = pECUsr->get_code(&_code);
		if(FAILED(hres)) _code = 0;

		hres = pECUsr->get_description(&bstrdesc);
		CDATAPACK::BSTR2WString(bstrdesc, _description);
		hres = pECUsr->get_scope(&bstrscope);
		CDATAPACK::BSTR2WString(bstrscope, _scope);
		hres = pECUsr->get_interfaceName(&bstrinterfacename);
		CDATAPACK::BSTR2WString(bstrinterfacename, _interfaceName);
		hres = pECUsr->get_operation(&bstroperation);
		CDATAPACK::BSTR2WString(bstroperation, _operation);
		hres = pECUsr->get_moreInfo(&bstrmoreinfo);
		CDATAPACK::BSTR2WString(bstrmoreinfo, _moreInfo);
		::SysFreeString(bstrdesc);
		::SysFreeString(bstrscope);
		::SysFreeString(bstrinterfacename);
		::SysFreeString(bstroperation);
		::SysFreeString(bstrmoreinfo);
	}
	if(pECUsr) pECUsr->Release();
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
CCapeUser::~CCapeUser(){
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::get_code(long& code){
	code = _code;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::get_description(std::wstring& description){
	description = _description;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::add_description(const wchar_t* description){
	_description += description;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::set_description(const std::wstring& description){
	_description = description;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::get_scope(std::wstring& scope){
	scope = _scope;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::set_scope(const std::wstring& scope){
	_scope = scope;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::get_interfaceName(std::wstring& interfaceName){
	interfaceName = _interfaceName;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::set_interfaceName(const std::wstring& interfaceName){
	_interfaceName = interfaceName;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::get_operation(std::wstring& operation){
	operation = _operation;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::set_operation(const std::wstring& operation){
	_operation = operation;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::get_moreInfo(std::wstring& moreInfo){
	moreInfo = _moreInfo;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeUser::set_moreInfo(const std::wstring& moreInfo){
	_moreInfo = moreInfo;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CCapeUser::get_name(std::wstring& name){
	name = _name;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CCapeUser::set_name(const std::wstring& name){
	_name = name;
}

} //End namespace R2PException
