// CCapeException.h: interface for the CCapeException class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(__CWARNING_H__)
#define __CWARNING_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <comdef.h>
#include <string>
#include <list>
#include "CCapeWarning.h"
#include "CMyException.h"

namespace R2PException{
	// ======================================================================================
	/// <summary>CapeOpen Warning handling class.</summary> 
	class CWarning
	{
	public:

		/// <summary>Constructor</summary>
		CWarning();

		/// <summary>Destructor</summary>
		virtual ~CWarning();

		/// <summary>Add a warning to the list</summary>
		/// <param name="name">A Short description of the warning</param>
		/// <param name="desc">Description of the warning</param>
		/// <param name="interfaceName">The name of the interface where the warning is thrown</param>
		/// <param name="userOperation">The name of the operation where the warning is thrown</param>
		/// <param name="hr">Warning code</param>
		/// <returns>void</returns>
		virtual void AddWarning(const std::wstring& name, const std::wstring& desc, const std::wstring& interfaceName, const std::wstring& userOperation, HRESULT hr = 0);

		/// <summary>Add a warning to the list</summary>
		/// <param name="pWarn">A Warning</param>
		/// <returns>void</returns>
		/// <remarks>the pWarn argument will not be reallocated: you do not have to free it.</remarks>
		virtual void AddWarning(CCapeWarning* pWarn);

		/// <summary>Add a warning to the list from exception</summary>
		/// <param name="pErr">error to convert into warning</param>
		/// <returns>void</returns>
		virtual void AddWarning(CMyException& herr);

		/// <summary>Get top warning of the list</summary>
		/// <returns>CCapeUser: pointer to a class object which is filled with useful information</returns>
		/// <remarks>The top warning is removed from the list. Client who gets it back has to free the pointer</remarks>
		CCapeWarning* RetrieveOne();

		/// <summary>Get all warnings</summary>
		/// <returns>list of CCapeUser: pointer to a list of class object, all filled with useful information</returns>
		/// <remarks>All warnings are removed from the list. Client who gets it back has to free the list and the list itself</remarks>
		std::list<CCapeWarning*>* RetrieveAll();

		/// <summary>Count the number of warnings</summary>
		/// <returns>unsinged int: the number of warning</returns>
		size_t Count();

	protected:
		std::list<CCapeWarning*>* _lstWarn;
	};
}
#endif // __CWARNING_H__
