// RegisterInformation.h: interface for the CRegisterInformation class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_REGISTERINFORMATION_H__D96059D8_7DCC_4423_9E16_3B059DCE66FC__INCLUDED_)
#define AFX_REGISTERINFORMATION_H__D96059D8_7DCC_4423_9E16_3B059DCE66FC__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#pragma warning (disable: 4251)
#pragma warning (disable :4530)
#include <atlbase.h>
#include <comutil.h>
#include <string>
#include <CMyException.h>
#include <ExceptionUtils.h>

#if !defined(_UNICODE)
#include "CDATAPACK.h"
#endif


/** CRegisterInformation class: class for retreiving configuration information

@author P. ROUX- TINA Project - Institut Franais du Ptrole - 2005 */
// ======================================================================================

/// <summary>Wrapper to windows registry: specific to R2P</summary>
class __declspec(dllexport) CRegisterInformation{
public:

	/// <summary>Get back trace directory</summary>
	/// <param name="wsXMLDir_out">path to the hydro XML file</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getHydroXML(std::wstring& wsXMLDir_out) throw(...);

	/// <summary>Get path to pipe DTD</summary>
	/// <param name="outRes">path to pipe dtd</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getPath2PipeDTD(std::wstring& outRes) throw(...);

	/// <summary>Get path to hydro DTD</summary>
	/// <param name="outRes">path to hydro dtd</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getPath2HydroDTD(std::wstring& outRes) throw(...);

	/// <summary>Get path to ip DTD</summary>
	/// <param name="outRes">path to ip dtd</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getPath2IPDTD(std::wstring& outRes) throw(...);

	/// <summary>Get back trace directory</summary>
	/// <param name="wsTrDir_out">directory where to trace</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getTraceDirectory(std::wstring& wsTrDir_out) throw(...);

	/// <summary>Get back trace mode</summary>
	/// <param name="wsTrMode_out">mode to trace</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getTraceMode(std::wstring& wsTrMode_out) throw(...);

	/// <summary>Get back Olga hydro path directory</summary>
	/// <param name="wsDirectory_out">Directory</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getOlgaPath(std::wstring& wsDirectory_out) throw(...);
	
	/// <summary>Get back initialisation mode</summary>
	/// <param name="wsInitMode_out">mode of initialisiation</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getInitialisationMode(std::wstring& wsInitMode_out) throw(...);

	/// <summary>Get back liquid phase name</summary>
	/// <param name="wsLiquidPhase_out">liquid pahse name</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getLiquidPhaseName(std::wstring& wsLiquidPhase_out) throw(...);

	/// <summary>Get back vapor phase name</summary>
	/// <param name="wsVaporPhase_out">vapor phase name</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getVaporPhaseName(std::wstring& wsVaporPhase_out) throw(...);

	/// <summary>Get back water phase name</summary>
	/// <param name="wsWaterPhase_out">water phase name</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getWaterPhaseName(std::wstring& wsWaterPhase_out) throw(...);

	/// <summary>Get back the filename containing mixture with water for testing purposes</summary>
	/// <param name="wsTestWaterMixture_out">MixtureFileName</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getTestWaterMixtureFileName(std::wstring& wsTestWaterMixture_out) throw(...);

	/// <summary>Get back the filename of mixture</summary>
	/// <param name="wsMixtureFile_out">MixtureFileName</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getMixtureFileName(std::wstring& wsMixtureFile_out) throw(...);

	/// <summary>Get back Mixture File Name for testing purposes</summary>
	/// <param name="wsTestMixtureFile_out">MixtureFileName</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getTestMixtureFileName(std::wstring& wsTestMixtureFile_out) throw(...);
	
	/// <summary>Get back horizontal pipe file name</summary>
	/// <param name="wsHPipeFile_out">fileName</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getHPipeFileName(std::wstring& wsHPipeFile_out) throw(...);

	/// <summary>Get back vertical pipe file name</summary>
	/// <param name="wsVPipeFile_out">fileName</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getVPipeFileName(std::wstring& wsVPipeFile_out) throw(...);
	
	/// <summary>Get back License file directory</summary>
	/// <param name="wsLicenseDir_out">directory where to find license</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getLicenseDirectory(std::wstring& wsLicenseDir_out) throw(...);

	/// <summary>Get back the directory of useful file to test pipe</summary>
	/// <param name="wsDataDir_out">directory where to find data file</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getTestDataDirectory(std::wstring& wsDataDir_out) throw(...);

	/// <summary>Get back the thermo directory</summary>
	/// <param name="wsThermoDir_out">directory where to find thermo files</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getThermoDirectory(std::wstring& wsThermoDir_out) throw(...);

	/// <summary>Get back indiss working directory</summary>
	/// <param name="wsWorkingDir_out">directory where INDISS is launched from</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void getWorkingDirectory(std::wstring& wsWorkingDir_out) throw(...);

private:
	/// <summary>Get back string in registry</summary>
	/// <param name="ws_out">the string to get back</param>
	/// <param name="wsPath">Path in registry</param>
	/// <param name="wsKey">Key in registry</param>
	/// <param name="dwType">Type of the Key</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static inline void getRegistry(std::wstring& ws_out, _TCHAR* wsPath, _TCHAR* wsKey, DWORD dwType) throw(...);
};
/****************************************************************************************************************************
*
*		Inline method
****************************************************************************************************************************/
void CRegisterInformation::getRegistry(std::wstring& ws_out, _TCHAR* wsPath, _TCHAR* wsKey, DWORD dwType){
	HKEY hKey = NULL;
	long lResult = ERROR_SUCCESS;
	unsigned long cbValue = 0;
	
	try{
		lResult = RegOpenKeyEx(HKEY_CURRENT_USER, wsPath, 0, KEY_READ, &hKey);
		R2PException::Utils::ThrowException(lResult == ERROR_SUCCESS, 0, L"", L"");
	
		//Get back length of the string
		lResult = RegQueryValueEx(hKey, wsKey, NULL, &dwType, NULL, &cbValue);
		R2PException::Utils::ThrowException(lResult == ERROR_SUCCESS, 0, L"", L"");
		
//Project properties: Unicode or multibyte
#if defined(_UNICODE)
		wchar_t* tszSrcName = (wchar_t*)alloca(sizeof(wchar_t)*(cbValue));
#else
		//default behavior: _MBCS or ???
		char* tszSrcName = (char*)alloca(sizeof(char)*(cbValue));
#endif

		//Get Value
		lResult = RegQueryValueEx(hKey, wsKey, NULL, &dwType, (LPBYTE)tszSrcName, &cbValue);
		R2PException::Utils::ThrowException(lResult == ERROR_SUCCESS, 0, L"", L"");

		RegCloseKey(hKey);
#if defined(_UNICODE)
		ws_out = tszSrcName;
#else
		//default behavior: _MBCS or ???
		CDATAPACK::String2WString(tszSrcName, ws_out);
#endif

	}
	catch(...){
		if(hKey) RegCloseKey(hKey);
		std::wstring msg = L"Try to access key ";
#if defined(_UNICODE)
		if(wsPath)
			msg += wsPath;
		else
			msg += L"(null)";
#else
		std::wstring tmp;
		CDATAPACK::String2WString(wsPath, tmp);
		msg += tmp;
#endif
		msg += L"\\";
#if defined(_UNICODE)
		if(wsKey)
			msg += wsKey;
		else
			msg += L"(null)";
#else
		CDATAPACK::String2WString(wsPath, tmp);
		msg += tmp;
#endif
		msg +=  L" in registry but it does not exist.\n";
		throw R2PException::CMyException(REGDB_E_READREGDB, L"Registry Error", msg.c_str());
	}
}

#endif // !defined(AFX_REGISTERINFORMATION_H__D96059D8_7DCC_4423_9E16_3B059DCE66FC__INCLUDED_)
