#ifndef TNT_ARRAY2D_UTILS_IFP_H
#define TNT_ARRAY2D_UTILS_IFP_H

/* Added Array2D functions.

   Provided by the CHELP library : wwwinfo.cern.ch/asd/lhc++/chelp/index.html */


#include "tnt.h"


namespace TNT
{

// ======================================================================================
template <class T>
int dfinv_Array2D(Array2D<T> &A, Array1D<int> &ir)
{
	const int DIM = A.dim1();

	double s31, s32, s33, s34;

	//T* m11 = &A.data_[0];
	T* m11 = A[0];
	T* m12 = m11 + 1;
	T* m21 = m11 + DIM;
	T* m22 = m12 + DIM;

	*m21 = -(*m22) * (*m11) * (*m21);
	*m12 = -(*m12);

	if (DIM > 2) {
		//T* mi    = &A.data_[0] + 2*DIM;
		//T* mii   = &A.data_[0] + 2*DIM + 2;
		//T* mimim = &A.data_[0] + DIM+1;
		T* mi    = A[0] + 2*DIM;
		T* mii   = A[0] + 2*DIM + 2;
		T* mimim = A[0] + DIM+1;

		for (int i=3; i<=DIM; i++) {
			int im2 = i - 2;
			//T* mj  = &A.data_[0];
			T* mj  = A[0];
			T* mji = mj + i-1;
			T* mij = mi;
			for (int j=1; j<=im2; j++) {
				s31 = 0.;
				s32 = *mji;
				T* mkj  = mj + j-1;
				T* mik  = mi + j-1;
				T* mjkp = mj + j;
				T* mkpi = mj + DIM + i-1;
				for (int k=j; k<=im2; k++) {
					s31  += (*mkj)      * (*(mik++));
					s32  += (*(mjkp++)) * (*mkpi);
					mkj  += DIM;
					mkpi += DIM;
				}
				*mij = -(*mii) * (((*(mij-DIM))) * ((*(mii-1)))+(s31));
				*mji = -s32;
				mj  += DIM;
				mji += DIM;
				mij++;
			}
			*(mii-1)   = -(*mii) * (*mimim) * (*(mii-1));
			*(mimim+1) = -(*(mimim+1));
			mi        += DIM;
			mimim     += DIM+1;
			mii       += DIM+1;
		}
	}

	//T* mi  = &A.data_[0];
	//T* mii = &A.data_[0];
	T* mi  = A[0];
	T* mii = A[0];
	for (int i=1; i<DIM; i++) {
		int ni = DIM - i;
		T* mij = mi;
		int j;
		for (j=1; j<=i; j++) {
			s33 = *mij;
			T* mikj = mi  + DIM + j-1;
			T* miik = mii + 1;
			T* min_end = mi + DIM;
			for ( ; miik<min_end; ) {
				s33  += (*mikj) * (*(miik++));
				mikj += DIM;
			}
			*(mij++) = s33;
		}

		for (j=1; j<=ni; j++) {
			s34 = 0.;
			T* miik = mii + j;
			T* mikij = mii + j*DIM + j;
			for (int k=j; k<=ni; k++) {
				s34   += *mikij * (*(miik++));
				mikij += DIM;
			}
			*(mii+j) = s34;
		}
		mi  += DIM;
		mii += DIM+1;
	}

	int nxch = ir[DIM];
	if (nxch == 0) return 0;

	for (int mm=1; mm<=nxch; mm++) {
		int k  = nxch - mm + 1;
		int ij = ir[k];
		int i  = ij >> 12;
		int j  = ij%4096;
		//T* mki = &A.data_[0] + i-1;
		//T* mkj = &A.data_[0] + j-1;
		T* mki = A[0] + i-1;
		T* mkj = A[0] + j-1;
		for (k=1; k<=DIM; k++) {
			double ti = *mki;
			*mki = *mkj;
			*mkj = ti;
			mki += DIM;
			mkj += DIM;
		}
	}

	return 0;
}


// ======================================================================================
template <class T>
int dfact_Array2D(Array2D<T> &A, double &det, Array1D<int> &ir)
{
	const int DIM = A.dim1();
	
	double tf;
	const double g1 = 1.e-19;
	const double g2 = 1.e+19;

	double p, q, t;
	double s11, s12;

	const double EPSILON = 1.e-10;

	int normal = 0;
	int imposs =-1;
	int jrange = 0;
	int jover  = 0;
	int junder = 0;
	int ifail = normal;
	int jfail = jrange;

	int nxch  = 0;
	det = 1.;

	//T*   mj  = &A.data_[0];
	T*   mj  = A[0];
	T*   mjj = mj;
	for (int j=1; j<=DIM; j++) {
		int k = j;
		p = fabs(*mjj);

		if (j != DIM) {
			T* mij = mj + DIM + j-1;
			for (int i=j+1; i<=DIM; i++) {
				q = fabs(*mij);
				if (q > p) {
					k = i;
					p = q;
				}
				mij += DIM;
			}
			if (k == j)  {
				if (p < EPSILON) {
					det = 0.;
					ifail = imposs;
					return ifail;
				}
				det = -det;
			}
			T* mjl = mj;
			//T* mkl = &A.data_[0] + (k-1)*DIM;
			T* mkl = A[0] + (k-1)*DIM;
			for (int l=1; l<=DIM; l++) {
				tf       = *mjl;
				*(mjl++) = *mkl;
				*(mkl++) = tf;
			}
			nxch++;
			ir[nxch] = (((j)<<12) + (k));

		} else {
			if (p < EPSILON) {
				det = 0.;
				ifail = imposs;
				return ifail;
			}
		}

		det *= *mjj;
		*mjj = 1. / *mjj;
		t = fabs(det);
		if (t < g1) {
			det = 0.;
			if (jfail == jrange) jfail = junder;
		} else if (t > g2) {
			det = 1.;
			if (jfail == jrange) jfail = jover;
		}

		if (j != DIM) {
			T* mk   = mj + DIM;
			T* mkjp = mk + j;
			T* mjk  = mj + j;
			for (k=j+1; k<=DIM; k++) {
				s11 = -(*mjk);
				s12 = -(*mkjp);
				if (j != 1) {
					//T* mik  = &A.data_[0] + k-1;
					//T* mijp = &A.data_[0] + j;
					T* mik  = A[0] + k-1;
					T* mijp = A[0] + j;
					T* mki  = mk;
					T* mji  = mj;
					for (int i=1; i<j; i++) {
						s11  += (*mik)  * (*(mji++));
						s12  += (*mijp) * (*(mki++));
						mik  += DIM;
						mijp += DIM;
					}
				}
				*(mjk++) = -s11 * (*mjj);
				*(mkjp)  = -( ((*(mjj+1))) * ((*(mkjp-1))) + (s12) );
				mk      += DIM;
				mkjp    += DIM;
			}
		}

		mj  += DIM;
		mjj += DIM+1;
	}

	if (nxch%2 == 1) det = -det;
	if (jfail != jrange) det = 0.;
	ir[DIM] = nxch;
	return 0;
}


// ======================================================================================
/** @brief Invert the square matrix Array2D
	@param A The matrix to invert (must be square) */
template <class T>
void invert(Array2D<T> &A)
{
	assert (A.dim1() == A.dim2());

	Array1D<int> ir(A.dim1()+1, 0);
	double       det;

	dfact_Array2D(A, det, ir);
	dfinv_Array2D(A, ir);
}


// ======================================================================================
/** @brief Invert a square matrix Array2D
	@param A The matrix to invert (must be square)
	@return The invert A-1 */
template <class T>
Array2D<T> inverse(const Array2D<T> &A)
{
	Array2D<T> mTmp(A.copy());
	invert(mTmp);
	return mTmp;
}


} // namespace TNT

#endif
